import 'package:flutter/material.dart';
import 'package:vector_math/vector_math_64.dart' show Quaternion, Vector3;
import 'dart:math' as math;
import 'dart:async' show Timer;


class Exo2 extends StatefulWidget {
  const Exo2({super.key});

  @override
  State<Exo2> createState() => _Exo2();
}

class _Exo2 extends State<Exo2> {
  double rx = 0.2;
  double ry = 0.5;
  double rz = 0.0;
  bool? mirror = false;
  double sc = 1.0;

  final d = Duration(milliseconds: 50);
  Timer? timer;
  bool sc_direction = false;

  void animate(Timer t) {
    setState(() {
      rx += 0.055;
      ry += 0.025;
      rz += 0.01;
      sc += sc_direction ? -0.03 : 0.03;
      if(sc > 2)
        sc_direction = true;
      else if(sc < 0.7)
        sc_direction = false;

      rx = rx > 2 * math.pi ? 0 : rx;
      ry = ry > 2 * math.pi ? 0 : ry;
      rz = rz > 2 * math.pi ? 0 : rz;
    });
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: "Exo2",
      home: Scaffold(
        appBar: AppBar(leading: BackButton(onPressed: () => Navigator.of(context).pop()),
        title: const Text('Slider')),
        body: SingleChildScrollView(child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            Container(
              clipBehavior: Clip.hardEdge,
              decoration: const BoxDecoration(color: Colors.white),
              child: LayoutBuilder(
                builder: (context, constraints) {
                  return Transform(
                    alignment: Alignment.center,
                    transform: Matrix4.compose(Vector3(0,0,0), Quaternion.euler(rx, ry, rz), Vector3(sc, sc, sc)),
                    child: Center(child: Image.network('https://picsum.photos/512/1024',
                        width: constraints.maxHeight,
                        height: constraints.maxWidth,
                        fit: BoxFit.contain,
                    )),
                  );
                }
              ),
            ),
            Text('Rotation arround x'),
            Slider(
              min: 0,
              max: 2 * math.pi,
              value: rx,
              label: rx.round().toString(),
              onChanged: (double value) {
                setState(() {
                  rx = value;
                });
              },
            ),
            Text('Rotation arround y'),
            Slider(
              min: 0,
              max: 2 * math.pi,
              value: ry,
              label: ry.round().toString(),
              onChanged: (double value) {
                setState(() {
                  ry = value;
                });
              },
            ),
            Text('Rotation arround z'),
            Slider(
              min: 0,
              max: 2 * math.pi,
              value: rz,
              label: rz.round().toString(),
              onChanged: (double value) {
                setState(() {
                  rz = value;
                });
              },
            ),
            Text('Mirror effect'),
            Checkbox(
              value: rx > math.pi,
              onChanged: (bool? val){
                setState(() {
                  rx += rx < math.pi ? math.pi : -math.pi;
                });
              },
            ),
            Text('Scale'),
            Slider(
              min: 0,
              max: 5,
              value: sc,
              label: sc.toString(),
              onChanged: (double val) {
                setState(() {
                  sc = val;
                });
              }
            ),
            TextButton(
              onPressed: () {
                if(timer == null || !(timer?.isActive ?? true)){
                  timer = new Timer.periodic(d, animate);
                } else {
                  timer?.cancel();
                }
              },
              child: const Text('Animate'),
            )
          ],
        )),
      ),
    );
  }
}
