import 'package:flutter/material.dart';
import 'dart:math' as math;

class Tile {
  Image image;
  Alignment alignment;

  Tile({required this.image, required this.alignment});

  Widget croppedImageTile(double widthFactor) {

    return FittedBox(
      fit: BoxFit.fill,
      child: ClipRect(
        child: Container(
          child: Align(
            alignment: this.alignment,
            widthFactor: widthFactor,
            heightFactor: widthFactor,
            child: image,
          ),
        ),
      ),
    );
  }
}

class Exo6 extends StatefulWidget {
  const Exo6({super.key});

  @override
  State<Exo6> createState() => _Exo6();
}

class _Exo6 extends State<Exo6> {
  var image = Image.network('https://picsum.photos/512');
  List<Widget> tiles = <Widget>[];
  double? size = 300;
  int colNb = 3;
  int emptyTileIndex = 0;

  @override
  void initState(){
    super.initState();

    image = Image.network('https://picsum.photos/512');

    int index = 0;
    emptyTileIndex = math.Random().nextInt(colNb * colNb);

    for(int i = 0; i < colNb; i++){
      for(int j = 0; j < colNb; j++){
        int indexcopy = index;

        if(index == emptyTileIndex){
          tiles.add(
            Flexible(child: Container(margin: EdgeInsets.all(10.0 / colNb)))
          );
        }else{
          Tile tile = Tile(image: image, alignment: 
            Alignment(-1 + 2 * j.toDouble() / (colNb.toDouble() - 1),
                      -1 + 2 * i.toDouble() / (colNb.toDouble() - 1))
          );

          Key k = UniqueKey();

          tiles.add(
            Flexible(key: k, child: Container(
              margin: EdgeInsets.all(10.0 / colNb),
              child: InkWell(
                child: tile.croppedImageTile(1 / colNb),
                onTap: () {
                  setState(() {
                    swapTiles(k);
                  });
                }
              )
            ))
          );
        }

        index++;
      }
    }
  }

  void swapTiles(Key k){
    var t = tiles.singleWhere((element) => element.key == k);
    tiles[emptyTileIndex] = t;
    tiles[tiles.indexOf(t)] = Flexible(child: Container(margin: EdgeInsets.all(10.0 / colNb)));
    emptyTileIndex = tiles.indexOf(t);
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text('Display a Tile as a Cropped Image'),
        centerTitle: true,
      ), 
      body: Column(children: [
        Container(
          margin: const EdgeInsets.all(10.0),
          child: SizedBox(
            height: MediaQuery.of(context).size.height * 0.7,
            width: MediaQuery.of(context).size.height * 0.7,
            child: GridView.builder(
              gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
                crossAxisCount: colNb,
              ),
              itemCount: colNb * colNb,
              itemBuilder: (BuildContext context, int index) {
                return tiles.elementAt(index);
              }
            )
          )
        ),
        Slider(
          value: colNb.toDouble(),
          min: 2,
          max: 6,
          onChanged: (double value) {
            setState(() {
              colNb = value.toInt();
            });
          }
        ),
      ]),
    );
  }
}
